import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

public class Greenberg extends Applet implements ActionListener
{

	public static int WIDTH=50,HEIGHT=50, RESOLUTION=10;

	Timer t;
	boolean paused;
	Button startButton, randButton, regButton, turbButton;
	TextField width, height, prob;
	int[][] cells, oldcells;
	int mode=0;

    public void init()
    {
		setSize(WIDTH*RESOLUTION+1, HEIGHT*RESOLUTION + 25);
		setLayout(new BorderLayout());
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new GridLayout(1,2));
		add("South", buttonPane);

		startButton = new Button("Start");
		startButton.addActionListener(this);
		buttonPane.add(startButton);
		randButton = new Button("Random");
		randButton.addActionListener(this);
		buttonPane.add(randButton);
		regButton = new Button("Regular");
		regButton.addActionListener(this);
		buttonPane.add(regButton);
		turbButton = new Button("Turbulent");
		turbButton.addActionListener(this);
		buttonPane.add(turbButton);

		width = new TextField("50", 2);
		width.addActionListener(this);
		buttonPane.add(width);
		height = new TextField("50", 2);
		height.addActionListener(this);
		buttonPane.add(height);

    }

    public void start()
    {
		paused = true;
		reset();
		t = new Timer(200, new ActionListener() {
			public void actionPerformed(ActionEvent e){
				tick();
		}});
		t.start();
    }

    public void dosize()
    {
		try
		{
			WIDTH = Integer.parseInt(width.getText());
		}
		catch (NumberFormatException e)
		{
			WIDTH = 50;
		}
		if (WIDTH<=0) WIDTH=50;
		if (WIDTH>50) WIDTH=50;
		try
		{
			HEIGHT = Integer.parseInt(height.getText());
		}
		catch (NumberFormatException e)
		{
			HEIGHT = 50;
		}
		if (HEIGHT<=0) HEIGHT=50;
		if (HEIGHT>50) HEIGHT=50;
		resize(Math.max(WIDTH*RESOLUTION+1, 251), HEIGHT*RESOLUTION + 25);
	}

	public void reset()
	{
		dosize();
		cells=new int[WIDTH][HEIGHT];
		oldcells=new int[WIDTH][HEIGHT];
		switch(mode)
		{
		case 0:
			cells[WIDTH/4][HEIGHT/2]=1;
			cells[WIDTH/4][HEIGHT/2+1]=2;
			break;
		case 1:
			cells[WIDTH/4][HEIGHT/2]=1;
			break;
		case 2:
			cells[WIDTH/2][HEIGHT/2]=1;
			cells[WIDTH/3][HEIGHT/2]=1;
			cells[WIDTH/2][HEIGHT/3]=1;
			break;
		}

		repaint();
	}

    public void destroy()
    {
		//
    }

    public void stop() {
		//
    }

    public String getAppletInfo()
    {
        return "Dissipative Cellular Automaton";
    }

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == startButton)
		{
			if (paused)
			{
				paused = false;
				startButton.setLabel("Pause");
			} else {
				paused = true;
				startButton.setLabel("Start");
			}
		}
		else if (e.getSource() == randButton)
		{
			mode=0;
			paused = true;
			startButton.setLabel("Start");
			reset();
		}
		else if (e.getSource() == regButton)
		{
			mode=1;
			paused = true;
			startButton.setLabel("Start");
			reset();
		}
		else if (e.getSource() == turbButton)
		{
			mode=2;
			paused = true;
			startButton.setLabel("Start");
			reset();
		}
	}

	public void paint(Graphics go) {
		Image im=new BufferedImage(WIDTH*RESOLUTION+2,HEIGHT*RESOLUTION+2,BufferedImage.TYPE_INT_RGB);
		Graphics g=im.getGraphics();
		g.setColor(Color.BLACK);
		g.fillRect(0,0,WIDTH*RESOLUTION+2,HEIGHT*RESOLUTION+2);
		for (int i=0;i<cells.length;i++)
		{
			for (int j=0;j<cells[0].length;j++)
			{
				int jr=j*RESOLUTION+1, ir=i*RESOLUTION+1;
				switch(cells[i][j])
				{
					case 0:
						g.setColor(Color.WHITE);
						break;
					case 1:
						g.setColor(Color.BLACK);
						break;
					case 2:
						g.setColor(Color.GRAY);
						break;
				}
				g.fillRect(ir,jr,RESOLUTION,RESOLUTION);
				g.setColor(Color.WHITE);
				g.drawLine(ir+RESOLUTION-1, jr, ir+RESOLUTION-1, jr+RESOLUTION-1);
				g.drawLine(ir, jr+RESOLUTION-1, ir+RESOLUTION-1, jr+RESOLUTION-1);
			}
		}
		go.drawImage(im,0,0,null);
	}
	public void update(Graphics go) {
		paint(go);
	}

	private int r(int a) {
		switch (a) {
			case 1:
			return 2;
			default:
			return 0;
		}
	}
	private int d(int a, int y1, int y2, int x1, int x2) {
		if (a==0)
			return (y1==1||y2==1||x1==1||x2==1)?1:0;
		else return 0;
	}

	public void tick()
	{
		if (paused)
			return;
		int[][] new_cells=new int[WIDTH][HEIGHT];

		for (int i=0; i<WIDTH; ++i)
			for (int j=0; j<HEIGHT; ++j)
			{
				int[] nbd = new int[8];
				nbd[0] = (i>0)        ? (cells[i-1][j]) : cells[WIDTH-1][j];
				nbd[1] = (i<WIDTH-1)  ? (cells[i+1][j]) : cells[0][j];
				nbd[2] = (j>0)        ? (cells[i][j-1]) : cells[i][HEIGHT-1];
				nbd[3] = (j<HEIGHT-1) ? (cells[i][j+1]) : cells[i][0];

				new_cells[i][j]=(r(cells[i][j])+d(cells[i][j],nbd[0],nbd[1],nbd[2],nbd[3])-oldcells[i][j])%3;

				if (new_cells[i][j]<0) new_cells[i][j]+=3; //correct for negative
			}
		oldcells=cells;
		cells=new_cells;
		repaint();
	}

}
