import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

public class Sandpile extends Applet implements ActionListener
{

	public static int WIDTH=50,HEIGHT=50, RESOLUTION=10;
	public static final int THRESHOLD = 4;

	Timer t;
	boolean paused;
	Button startButton, resetButton, flatButton;
	TextField width, height;
	int[][] cells;
	boolean[][] hit;
	boolean in_avalanche, end_avalanche;

    public void init()
    {
		setSize(WIDTH*RESOLUTION+1, HEIGHT*RESOLUTION + 25);
		setLayout(new BorderLayout());
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new GridLayout(1,2));
		add("South", buttonPane);

		startButton = new Button("Start");
		startButton.addActionListener(this);
		buttonPane.add(startButton);
		resetButton = new Button("Random");
		resetButton.addActionListener(this);
		buttonPane.add(resetButton);
		flatButton = new Button("Flat");
		flatButton.addActionListener(this);
		buttonPane.add(flatButton);

		width = new TextField("50", 2);
		width.addActionListener(this);
		buttonPane.add(width);
		height = new TextField("50", 2);
		height.addActionListener(this);
		buttonPane.add(height);

    }

    public void start()
    {
		paused = true;
		reset();
		t = new Timer(100, new ActionListener() {
			public void actionPerformed(ActionEvent e){
				tick();
		}});
		t.start();
    }

    public void dosize()
    {
		try
		{
			WIDTH = Integer.parseInt(width.getText());
		}
		catch (NumberFormatException e)
		{
			WIDTH = 50;
		}
		if (WIDTH<=0) WIDTH=50;
		if (WIDTH>50) WIDTH=50;
		try
		{
			HEIGHT = Integer.parseInt(height.getText());
		}
		catch (NumberFormatException e)
		{
			HEIGHT = 50;
		}
		if (HEIGHT<=0) HEIGHT=50;
		if (HEIGHT>50) HEIGHT=50;
		resize(Math.max(WIDTH*RESOLUTION+1, 251), HEIGHT*RESOLUTION + 25);
	}

	public void reset()
	{
		dosize();
		cells=new int[WIDTH][HEIGHT];
		for(int i=0; i<cells.length;i++)
			for(int j=0; j<cells[0].length;j++)
				cells[i][j]=(int)(Math.random()*4);//3;
		hit = new boolean[WIDTH][HEIGHT];
		in_avalanche = false;
		end_avalanche = false;
		repaint();
	}

	public void flatten()
	{
		dosize();
		cells=new int[WIDTH][HEIGHT];
		for(int i=0; i<cells.length;i++)
			for(int j=0; j<cells[0].length;j++)
				cells[i][j]=3;
		hit = new boolean[WIDTH][HEIGHT];
		in_avalanche = false;
		end_avalanche = false;
		repaint();
	}

    public void destroy()
    {
		//
    }

    public void stop() {
		//
    }

    public String getAppletInfo()
    {
        return "Abelian Sandpile Model";
    }

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == startButton)
		{
			if (paused)
			{
				paused = false;
				startButton.setLabel("Pause");
			} else {
				paused = true;
				startButton.setLabel("Start");
			}
		}
		else if (e.getSource() == resetButton)
		{
			reset();
			paused = true;
			startButton.setLabel("Start");
		}
		else if (e.getSource() == flatButton)
		{
			flatten();
			paused = true;
			startButton.setLabel("Start");
		}
	}

	private void avalanche()
	{
		int[][] newcells = new int[WIDTH][HEIGHT];
		for (int i=0; i<WIDTH; ++i)
			System.arraycopy(cells[i], 0, newcells[i], 0, HEIGHT);
		in_avalanche=false;
		for (int i=0; i<(WIDTH); ++i)
		for (int j=0; j<(HEIGHT); ++j)
			if (cells[i][j]>=THRESHOLD)
			{
				in_avalanche=true;
				hit[i][j]=true;
				newcells[i][j]-=4;
				if (i>0) newcells[i-1][j]++;
				if (i<WIDTH-1) newcells[i+1][j]++;
				if (j>0) newcells[i][j-1]++;
				if (j<HEIGHT-1) newcells[i][j+1]++;
			}
		cells = newcells;
	}

	public void tick()
	{
		if (paused)
			return;
		if (in_avalanche)
		{
			avalanche();
			repaint();
			return;
		}
		else if (end_avalanche)
		{
			hit = new boolean[WIDTH][HEIGHT];
			end_avalanche=false;
			repaint();
		}
		int x = (int)(Math.random()*WIDTH);
		int y = (int)(Math.random()*HEIGHT);
		cells[x][y]++;
		if (cells[x][y]>=THRESHOLD)
		{
			in_avalanche = true;
			end_avalanche=true;
		}
		repaint();
	}


	public void paint(Graphics go) {
		Image im=new BufferedImage(WIDTH*RESOLUTION+2,HEIGHT*RESOLUTION+2,BufferedImage.TYPE_INT_RGB);
		Graphics g=im.getGraphics();
		g.setColor(Color.WHITE);
		g.fillRect(0,0,WIDTH*RESOLUTION+2,HEIGHT*RESOLUTION+2);
		for (int i=0;i<cells.length;i++)
		{
			for (int j=0;j<cells[0].length;j++)
			{
				int jr=j*RESOLUTION+1, ir=i*RESOLUTION+1;
				switch(cells[i][j])
				{
					case 0:
						g.setColor(Color.CYAN);
						break;
					case 1:
						g.setColor(Color.GREEN);
						break;
					case 2:
						g.setColor(Color.YELLOW);
						break;
					case 3:
						g.setColor(Color.ORANGE);
						break;
					case 4:
					default:
						g.setColor(Color.RED);
						break;
				}
				g.fillRect(ir,jr,RESOLUTION,RESOLUTION);
				if (hit[i][j] || cells[i][j]>=4)
				{
					g.setColor(Color.BLACK);
					g.drawRect(ir-1,jr-1,RESOLUTION,RESOLUTION);
				} else {
					g.setColor(Color.WHITE);
					g.drawLine(ir+RESOLUTION-1, jr, ir+RESOLUTION-1, jr+RESOLUTION-1);
					g.drawLine(ir, jr+RESOLUTION-1, ir+RESOLUTION-1, jr+RESOLUTION-1);
				}
			}
		}
		go.drawImage(im,0,0,null);
	}
	public void update(Graphics go) {
		paint(go);
	}
}
